@@ -18,20 +18,26 @@
 #include "log.h"
 
 static int
-seafile_switch_account (const char *server, const char *username,
-                        const char *token,
-                        int is_pro, GError **error)
+seafile_add_account (const char *server, const char *username,
+                     const char *nickname,
+                     const char *token,
+                     const char *name,
+                     int is_pro, GError **error)
 {
-    if (!server || !username || !token) {
+    if (!server || !username || !token
+        || !name
+        ) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Empty Arguments");
         return -1;
     }
 
-    return seaf_repo_manager_switch_current_account (seaf->repo_mgr,
-                                                     server,
-                                                     username,
-                                                     token,
-                                                     (is_pro > 0));
+    return seaf_repo_manager_add_account (seaf->repo_mgr,
+                                          server,
+                                          username,
+                                          nickname,
+                                          token,
+                                          name,
+                                          (is_pro > 0));
 }
 
 /* static int */
@@ -50,14 +56,22 @@ seafile_switch_account (const char *server, const char *username,
 /* } */
 
 static int
-seafile_delete_account (const char *server, const char *username, int remove_cache, GError **error)
+seafile_delete_account (const char *server,
+                        const char *username,
+                        gboolean remove_cache,
+                        GError **error)
 {
-    if (!server || !username) {
+    if (!server || !username
+       ) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Empty Arguments");
         return -1;
     }
 
-    return seaf_repo_manager_delete_account (seaf->repo_mgr, server, username, remove_cache);
+    return seaf_repo_manager_delete_account (seaf->repo_mgr,
+                                             server,
+                                             username,
+                                             remove_cache,
+                                             error);
 }
 
 static int
@@ -194,7 +208,9 @@ seafile_enable_auto_sync (GError **error)
 }
 
 static char *
-seafile_get_path_sync_status (const char *repo_uname,
+seafile_get_path_sync_status (const char *server,
+                              const char *username,
+                              const char *repo_uname,
                               const char *path,
                               GError **error)
 {
@@ -207,7 +223,7 @@ seafile_get_path_sync_status (const char *repo_uname,
         return NULL;
     }
 
-    repo_id = seaf_repo_manager_get_repo_id_by_name (seaf->repo_mgr, repo_uname);
+    repo_id = seaf_repo_manager_get_repo_id_by_display_name (seaf->repo_mgr, server, username, repo_uname);
     if (!repo_id) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Invalid repo unique name");
         return NULL;
@@ -228,17 +244,6 @@ seafile_get_path_sync_status (const char *repo_uname,
     return status;
 }
 
-static char *
-seafile_get_category_sync_status (const char *category, GError **error)
-{
-    if (!category) {
-        g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Argument should not be null");
-        return NULL;
-    }
-
-    return seaf_sync_manager_get_category_sync_status (seaf->sync_mgr, category);
-}
-
 static json_t *
 seafile_get_sync_notification (GError **error)
 {
@@ -252,7 +257,7 @@ seafile_get_events_notification (GError **error)
 }
 
 static char*
-seafile_get_repo_id_by_uname (const char *repo_uname, GError **error)
+seafile_get_repo_id_by_uname (const char *server, const char *username, const char *repo_uname, GError **error)
 {
     char *repo_id;
 
@@ -261,7 +266,7 @@ seafile_get_repo_id_by_uname (const char *repo_uname, GError **error)
         return NULL;
     }
 
-    repo_id = seaf_repo_manager_get_repo_id_by_name (seaf->repo_mgr, repo_uname);
+    repo_id = seaf_repo_manager_get_repo_id_by_display_name (seaf->repo_mgr, server, username, repo_uname);
     if (!repo_id) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Invalid repo unique name");
         return NULL;
@@ -333,7 +338,7 @@ seafile_mark_file_locked (const char *repo_id, const char *path, GError **error)
         canon_path[len-1] = 0;
 
     ret = seaf_filelock_manager_mark_file_locked (seaf->filelock_mgr,
-                                                  repo_id, canon_path, FALSE);
+                                                  repo_id, canon_path, LOCKED_MANUAL);
 
     g_free (canon_path);
     return ret;
@@ -372,20 +377,20 @@ seafile_mark_file_unlocked (const char *repo_id, const char *path, GError **erro
 }
 
 static int
-seafile_cancel_download (const char *full_file_path, GError **error)
+seafile_cancel_download (const char *server, const char *user, const char *full_file_path, GError **error)
 {
     if (!full_file_path) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Argument should not be null");
         return -1;
     }
 
-    return file_cache_mgr_cancel_download (seaf->file_cache_mgr, full_file_path, error);
+    return file_cache_mgr_cancel_download (seaf->file_cache_mgr, server, user, full_file_path, error);
 }
 
 static json_t *
 seafile_list_sync_errors (GError **error)
 {
-    return seaf_sync_manager_list_sync_errors (seaf->sync_mgr);
+    return seaf_repo_manager_list_sync_errors (seaf->repo_mgr, 0, 50);
 }
 
 static int
@@ -439,6 +444,8 @@ seafile_get_enc_repo_list (GError **error)
         //used to determine whether the repo is in the current account when switch account.
         if (!display_name)
             continue;
+        json_object_set_new (obj, "server", json_string(repo->server));
+        json_object_set_new (obj, "username", json_string(repo->user));
         json_object_set_new (obj, "repo_id", json_string(repo->id));
         json_object_set_new (obj, "repo_display_name", json_string(display_name));
         if (repo->is_passwd_set)
@@ -459,6 +466,7 @@ static int
 seafile_set_enc_repo_passwd (const char *repo_id, const char *passwd, GError **error)
 {
     SeafRepo *repo;
+    int ret = 0;
 
     if (!repo_id || !passwd) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Argument should not be null");
@@ -471,16 +479,31 @@ seafile_set_enc_repo_passwd (const char *repo_id, const char *passwd, GError **e
         return -1;
     }
 
-    if (seafile_verify_repo_passwd (repo_id, passwd, repo->magic,
-                                    repo->enc_version, repo->salt) < 0) {
-        g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Wrong password");
+    if (repo->pwd_hash_algo) {
+        if (seafile_pwd_hash_verify_repo_passwd (repo->enc_version, repo_id, passwd,
+                                                 repo->salt, repo->pwd_hash,
+                                                 repo->pwd_hash_algo, repo->pwd_hash_params) < 0) {
+            g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Wrong password");
+            ret = -1;
+            goto out;
+        }
+
+    } else {
+        if (seafile_verify_repo_passwd (repo_id, passwd, repo->magic,
+                                        repo->enc_version, repo->salt) < 0) {
+            g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Wrong password");
+            ret = -1;
+            goto out;
+        }
     }
 
-    int ret = seaf_repo_manager_set_repo_passwd (seaf->repo_mgr, repo, passwd);
+    ret = seaf_repo_manager_set_repo_passwd (seaf->repo_mgr, repo, passwd);
     if (ret < 0) {
         g_set_error (error, SEAFILE_DOMAIN, SEAF_ERR_BAD_ARGS, "Failed to insert data into db");
+        goto out;
     }
 
+out:
     seaf_repo_unref (repo);
     return ret;
 }
@@ -496,6 +519,24 @@ seafile_clear_enc_repo_passwd (const char *repo_id, GError **error)
     return ret;
 }
 
+static int
+seafile_add_del_confirmation (const char *confirmation_id, int resync, GError **error)
+{
+    return seaf_sync_manager_add_del_confirmation (seaf->sync_mgr, confirmation_id, resync);
+}
+
+static json_t *
+seafile_get_account_by_repo_id (const char *repo_id, GError **error)
+{
+    return seaf_repo_manager_get_account_by_repo_id (seaf->repo_mgr, repo_id);
+}
+
+static char *
+seafile_ping (GError **error)
+{
+    return g_strdup ("pong");
+}
+
 #if 0
 
 static GObject *
@@ -618,9 +659,9 @@ register_rpc_service ()
     searpc_create_service ("seadrive-rpcserver");
 
     searpc_server_register_function ("seadrive-rpcserver",
-                                     seafile_switch_account,
-                                     "seafile_switch_account",
-                                     searpc_signature_int__string_string_string_int());
+                                     seafile_add_account,
+                                     "seafile_add_account",
+                                     searpc_signature_int__string_string_string_string_string_int());
 
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_delete_account,
@@ -710,12 +751,7 @@ register_rpc_service ()
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_get_path_sync_status,
                                      "seafile_get_path_sync_status",
-                                     searpc_signature_string__string_string());
-
-    searpc_server_register_function ("seadrive-rpcserver",
-                                     seafile_get_category_sync_status,
-                                     "seafile_get_category_sync_status",
-                                     searpc_signature_string__string());
+                                     searpc_signature_string__string_string_string_string());
 
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_get_sync_notification,
@@ -730,7 +766,7 @@ register_rpc_service ()
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_get_repo_id_by_uname,
                                      "seafile_get_repo_id_by_uname",
-                                     searpc_signature_string__string());
+                                     searpc_signature_string__string_string_string());
 
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_get_repo_display_name_by_id,
@@ -765,7 +801,7 @@ register_rpc_service ()
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_cancel_download,
                                      "seafile_cancel_download",
-                                     searpc_signature_int__string());
+                                     searpc_signature_int__string_string_string());
 
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_list_sync_errors,
@@ -796,6 +832,20 @@ register_rpc_service ()
                                      "seafile_clear_enc_repo_passwd",
                                      searpc_signature_int__string());
 
+    searpc_server_register_function ("seadrive-rpcserver",
+                                     seafile_add_del_confirmation,
+                                     "seafile_add_del_confirmation",
+                                     searpc_signature_int__string_int());
+    searpc_server_register_function ("seadrive-rpcserver",
+                                     seafile_get_account_by_repo_id,
+                                     "seafile_get_account_by_repo_id",
+                                     searpc_signature_json__string());
+
+    searpc_server_register_function ("seadrive-rpcserver",
+                                     seafile_ping,
+                                     "seafile_ping",
+                                     searpc_signature_string__void());
+
 #if 0
     searpc_server_register_function ("seadrive-rpcserver",
                                      seafile_generate_magic_and_random_key,
